#!/usr/bin/perl -w
##########################################################################
#
#   Copyright (c) 2003 Aymargeddon Development Team
#
#   This file is part of "Last days of Aymargeddon" 
#
#   Aymargeddon is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2 of the License, or (at your option)
#   any later version.
#
#   Aymargeddon is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#   more details.
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc., 675
#   Mass Ave, Cambridge, MA 02139, USA. 
#
###########################################################################
#

# generell Aymargeddon-specific functions
#


# TODO: sight
# TODO: move color calculation from map.epl in this module
#
#

use strict;
use FROGS::DataBase;
use FROGS::HexTorus;

package Aymargeddon;
@Aymargeddon::ISA = qw(DataBase);

sub new{
  my ($class,$game,$user) = @_;

  my $self = DataBase->new();
  $self->{-game} = $game;
  $self->{-user} = $user;

  bless($self,$class);
}

sub get_map{
  my $self = shift;

  unless (exists $self->{-map}){
    $self->{-map} = $self->read_map($self->{-game}, "TERRAIN,HOME,OCCUPANT,TEMPLE");
  }
  return $self->{-map};
}

sub get_size{
  my $self = shift;
  unless ($self->{-size}){
    my @size = $self->read_game($self->{-game},'SIZE');
    $self->{-size} = $size[0] ? $size[0] : die "could not find size";
  };

  return $self->{-size};
};

sub god_fight{
  my ($self,$loc_str) = @_;
  my @ret = $self->single_from_map('GOD_ATTACKER',$self->{-game},$loc_str);
  return $ret[0] ? 1 : 0;
}

sub earthling_fight{
  my ($self,$loc_str) = @_;
  my @ret = $self->single_from_map('ATTACKER',$self->{-game},$loc_str);
  return $ret[0] ? 1 : 0;
}

sub arc_present{
  my ($self,$loc_str) = @_;
  my $arks_ref = $self->read_mobile('ID','ARK',$self->{-game},$loc_str);
  my @arks = @$arks_ref;
  return $#arks+1;
}

sub avatar_present{
  my ($self,$loc_str) = @_;
  return $self->read_mobile('OWNER','AVATAR',$self->{-game},$loc_str);
}

sub mobiles_available{
  my ($self,$loc_str) = @_;
  my $fields = 'ID, TYPE, OWNER, ADORING, AVAILABLE, STATUS';
  return $self->read_mobile($fields,'',$self->{-game},$loc_str);
}

sub sight{
  my ($self,$loc_str) = @_;

  return 1 if $self->{-user} == -1; # admin sees all

  $self->generate_sight() unless $self->{-sight_map};

  return 1 if $self->{-sight_map}->{$loc_str};
}


sub generate_sight{
  my ($self) = @_;

  delete $self->{-sight_map};

  my $rel = $self->reverse_player_relations($self->{-game},$self->{-user});

  $self->sight_of_player($self->{-user});
  for my $player (keys %$rel){
    my $status = $rel->{$player};
    if($status eq 'FRIEND' or $status eq 'BETRAY'){
      $self->sight_of_player($player);
    }
  }
}

sub sight_of_player{
  my ($self,$player) = @_;

  my $map = HexTorus->new($self->get_size());

  my $selfmap = $self->get_map();

  for my $row (@$selfmap){
    my ($loc,$ter,$own,$occ,$temple) = @$row;
    next if $self->{-sight_map}->{$loc};

    if($own == $player or $occ == $player
       or @{$self->own_in_mobile($self->{-game},$loc,$player)}){
      $self->{-sight_map}->{$loc} = 1;
      my $location = Location->from_string($loc);
      my $dist = 1;
      $dist = 2 if $ter eq 'MOUNTAIN';
      my @neighbours = $map->distant_neighbours($location,$dist);
      for my $l (@neighbours){
	$self->{-sight_map}->{$l->to_string()} = 1;
      }
    }
  }
}

sub charname{
  my ($self,$player) = @_;
  return 'nobody' if $player < 1;
  my @list = $self->read_role($self->{-game}, $player, 'NICKNAME');
  return $list[0];
}

sub is_god{
  my ($self,$player) = @_;
  return 0 if $player < 1;
  my @role = $self->read_role($self->{-game}, $player, 'ROLE');
  return ($role[0] eq 'GOD');
}

sub relation_string{
  my ($self, $rel) = @_;
  return 'friend' if($rel eq 'FRIEND');
  return 'enemy' if($rel eq 'FOE');
  return 'allied' if($rel eq 'ALIIED');
  return 'betray' if($rel eq 'BETRAY');
  return 'neutral';
}

sub new_role{
  my($self,$role,$name,$gender,$desc) = @_;

  ($name, $gender, $desc, my $qrole) = $self->quote_all($name, $gender, $desc, $role);

  my $cond = 'GAME='.$self->{-game}." and NICKNAME=$name";
  return 0 if @{$self->select_array('ROLE','GAME',$cond)}; # error: dublicate name

  my @homes; # all possible homes for this role
  if($role ne 'OBSERVER'){
    @homes = $self->open_homes($role);
    return 0 if $#homes<0; # error: no home free
  }

  use Data::Dumper;

  # write in ROLE
  $self->insert_hash('ROLE',{'GAME' => $self->{-game},
			     'PLAYER' => $self->{-user},
			     'NICKNAME' => $name,
			     'ROLE' => $qrole,
			     'GENDER' => $gender,
			     'DESCRIPTION' => $desc});

  if($role eq 'OBSERVER'){
    $self->{-dbh}->commit();
    return 1;
  }

  # choose home:
  my $home = $homes[rand($#homes + 1)]->[0];

  if($role eq 'GOD'){
    # read actual default manapool from GAME
    my ($mana) = $self->read_game($self->{-game},'START_MANA');

    # write GAME, PLAYER, MANA in GOD
    $self->insert_hash('GOD',{'GAME' => $self->{-game},
			       'PLAYER' => $self->{-user},
			       'MANA' => $mana});

    # choose second home
    my $home2 = $home;
    if($#homes > 0){
      while($home2 eq $home){
	$home2 = $homes[rand($#homes + 1)]->[0];
      }
    }

    # change OWNER in MAP where LOCATION=$home or LOCATION=$home2
    ($home,$home2) = $self->quote_all($home,$home2);
    $self->update_hash('MAP',
		       "GAME=$self->{-game} AND ( LOCATION=$home OR LOCATION=$home2 )",
		       {'HOME' => $self->{-user}});

  }else{
    # write GAME, PLAYER in EARTHLING
    $self->insert_hash('EARTHLING',{'GAME' => $self->{-game},
				    'PLAYER' => $self->{-user}});

    # change OCCUPANT, OWNER in MAP where LOCATION=$home
    ($home) = $self->quote_all($home);
    $self->update_hash('MAP',
		       "GAME=$self->{-game} AND LOCATION=$home",
		       {'HOME' => $self->{-user},
			'OCCUPANT' => $self->{-user}});

    # change PLAYER of WARRIORS in MOBILE where LOCATION=$home
    $self->update_hash('MOBILE',
		       "GAME=$self->{-game} AND LOCATION=$home AND OWNER=-1",
		       {'OWNER' => $self->{-user}});
  }

  # TODO: write MESSAGE to all members of this game: new player with role!

  $self->{-dbh}->commit();
  return 1;
}

sub open_homes{
  my ($self,$role) = @_;
  my $terrain = ($role eq 'GOD') ? 'MOUNTAIN' : 'CITY';
  ($terrain) = $self->quote_all($terrain);
  my $cond = 'GAME='.$self->{-game}." AND HOME=-1 AND TERRAIN=$terrain";
  return @{$self->select_array('MAP','LOCATION',$cond)};
}

1;
