##########################################################################
#
#   Copyright (c) 2003 Aymargeddon Development Team
#
#   This file is part of
#   "FROGS" = Framework for Realtime Online Games of Strategy
#
#   FROGS is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2 of the License, or (at your option)
#   any later version.
#
#   FROGS is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#   more details.
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc., 675
#   Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################
#

#
# Generell database methods are gathered here.
use strict;
use DBI;
use FROGS::Config qw($conf);

package DataBase;

# the constructor connects to the DB
sub new{
  my $class = shift;
  my $self = { -source => $::conf->{-DB_SOURCE},
	       -user => $::conf->{-DB_USER},
	       -passwd => $::conf->{-DB_PASSWD},
	     };

  $self->{-dbh} = DBI->connect("dbi:$self->{-source}",
			       $self->{-user},
			       $self->{-passwd},
			       {'RaiseError' => 1,
				'AutoCommit' => 0});

  bless ($self,$class);
}

# returns an arrayref with the result of the given select statement
# TODO: forget it, use selectall_arrayref() from DBI instead.
sub select{
  my $self = shift;
  my $stmt = shift;

  return $self->{-dbh}->selectall_arrayref($stmt);
}

# assumes that a single row is returned from database
sub single_select{
  my ($self,$stmt) = @_;
  my $dat = $self->select($stmt);
  my @newdat = map {$_->[0]} @$dat;
  return \@newdat;
}

sub select_hash{
  my ($self, $table, $key, $fields, $cond) = @_;
  my $stmt = $fields ? "SELECT $key, $fields FROM $table"
                     : "SELECT $key FROM $table";
  $stmt .= " WHERE $cond" if $cond;
  return $self->{-dbh}->selectall_hashref($stmt, $key);
}

sub select_array{
  my ($self, $table, $fields, $cond) = @_;
  my $stmt = "SELECT $fields FROM $table";
  $stmt .= " WHERE $cond" if $cond;
  return $self->{-dbh}->selectall_arrayref($stmt);
}

sub read_map{
  my ($self, $game, $fields) = @_;
  return $self->{-dbh}->selectall_arrayref("SELECT LOCATION,$fields FROM MAP WHERE GAME=$game");
}

sub read_player_relations{
  my ($self, $game, $user) = @_;
  return $self->select_hash('ALLIANCE', 'OTHER', 'STATUS', "GAME=$game AND PLAYER=$user");
}

sub reverse_player_relations{
  my ($self, $game, $user) = @_;
  return $self->select_hash('ALLIANCE', 'PLAYER', 'STATUS', "GAME=$game AND OTHER=$user");
}

sub single_from_map {
  my ($self,$field,$game,$loc) = @_;
  my $qloc = $self->{-dbh}->quote($loc);
  my $stmt = "SELECT $field from MAP where GAME=$game AND LOCATION = $qloc";
  return $self->single_select($stmt);
}

sub read_mobile {
  my ($self,$fields,$type,$game,$loc) = @_;
  ($loc, $type) = $self->quote_all($loc, $type);
  my $cond = "GAME=$game AND LOCATION=$loc AND TYPE=$type";
  return $self->select_array('MOBILE', $fields, $cond);
}

sub own_in_mobile{
  my($self,$game,$loc,$player) = @_;
  my $qloc = $self->{-dbh}->quote($loc);
  my $cond = "GAME=$game AND LOCATION=$qloc AND (OWNER=$player OR ADORING=$player)";
  return $self->select_array('MOBILE','ID',$cond);
}

sub read_game{
  my ($self,$game,$field) = @_;
  my $stmt = "SELECT $field from GAME where GAME=$game";
  return $self->single_select($stmt);
}

sub find_first_free{
  my ($self,$table,$field) = @_;

  my $t = $self->select_array($table, $field);
  my @ids = sort {$a <=> $b} (map {$_->[0]} @$t);

  my $id = 1;
  for my $try (@ids){
    last if $id < $try;
    $id = $try + 1;
  }
  return $id;
}

sub new_account{
  my ($self,$login,$name,$email) = @_;

  ($login,$name,$email) = $self->quote_all($login,$name,$email);

  my $cond = "LOGIN=$login OR REALNAME=$name OR EMAIL=$email";
  my $habschon = $self->select_array('PLAYER','PLAYER',$cond);
  my @habschon = @$habschon;
  return 0 if @habschon;

  # generate new passwd
  my $pwd = "willkommen";
  my $qpwd = $self->{-dbh}->quote($pwd);

  # search first free player ID
  my $player = $self->find_first_free('PLAYER','PLAYER');

  # write new player
  my $insert = "INSERT INTO PLAYER (PLAYER,REALNAME,LOGIN,EMAIL,PASSWORD) ".
    "VALUES ($player,$name,$login,$email,$qpwd)";
  my $h = $self->{-dbh}->prepare($insert);
  $h->execute();
  $h->finish();
  $self->{-dbh}->commit();

  return $pwd;
}

sub authenticate{
  my ($self, $user, $pwd ) = @_;

  ($user,$pwd) = $self->quote_all($user,$pwd);

  my $stmt = "SELECT PLAYER FROM PLAYER WHERE LOGIN=$user AND PASSWORD=$pwd";
  my $auth = $self->single_select($stmt);
  return $auth->[0] if $auth;
  return 0;
}

sub quote_all{
  my ($self, @list) = @_;

  my @ret;
  for my $s (@list){
    push @ret, $self->{-dbh}->quote($s);
  }
  return @ret;
}

sub games_of_player{
  my ($self,$player) = @_;

  return ['1'];
}

# calls derived class!
sub open_games{
  my $self = shift;
  my $games = $self->select_array('GAME','GAME');

  my @log;
  for my $game (@$games){
    push @log, $game if $self->is_open($game);
  }
  return \@log;
}

1;


