#!/usr/bin/perl -w
##########################################################################
#
#   Copyright (c) 2003 Aymargeddon Development Team
#
#   This file is part of "Last days of Aymargeddon" 
#
#   Aymargeddon is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2 of the License, or (at your option)
#   any later version.
#
#   Aymargeddon is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#   more details.
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc., 675
#   Mass Ave, Cambridge, MA 02139, USA. 
#
###########################################################################
#

# generell Aymargeddon-specific functions
#


# TODO: sight
# TODO: move color calculation from map.epl in this module
#
#

use strict;
use FROGS::DataBase;
use FROGS::HexTorus;

package Aymargeddon;
@Aymargeddon::ISA = qw(DataBase);

sub new{
  my ($class,$game,$user) = @_;

  my $self = DataBase->new();
  $self->{-game} = $game;
  $self->{-user} = $user;

  bless($self,$class);
}

sub get_map{
  my $self = shift;

  unless (exists $self->{-map}){
    $self->{-map} = $self->read_map($self->{-game}, "TERRAIN,HOME,OCCUPANT,TEMPLE");
  }
  return $self->{-map};
}

sub get_size{
  my $self = shift;
  unless ($self->{-size}){
    my $size = $self->read_game($self->{-game},'SIZE');
    $self->{-size} = @$size ? $size->[0] : die "could not find size";
  };

  return $self->{-size};
};

sub god_fight{
  my ($self,$location_str) = @_;
  my $ret = $self->single_from_map('GOD_ATTACKER',$self->{-game},$location_str);
  return ($ret->[0]) ? 1 : 0;
}

sub earthling_fight{
  my ($self,$location_str) = @_;
  my $ret = $self->single_from_map('ATTACKER',$self->{-game},$location_str);
  return ($ret->[0]) ? 1 : 0;
}

sub arc_present{
  my ($self,$location_str) = @_;
  my $arks_ref = $self->read_mobile('ID','ARK',$self->{-game},$location_str);
  my @arks = @$arks_ref;
  return $#arks+1;
}

sub avatar_present{
  my ($self,$location_str) = @_;
  return $self->read_mobile('OWNER','AVATAR',$self->{-game},$location_str);
}


sub sight{
  my ($self,$loc_string) = @_;

  return 1 if $self->{-user} == -1; # admin sees all

  $self->generate_sight() unless $self->{-sight_map};

  return 1 if $self->{-sight_map}->{$loc_string};
}


sub generate_sight{
  my ($self) = @_;

  delete $self->{-sight_map};

  my $rel = $self->reverse_player_relations($self->{-game},$self->{-user});

  $self->sight_of_player($self->{-user});
#  for my $player (keys %$rel){
#    my $status = $rel->{$player};
#    if($status eq 'FRIEND' or $status eq 'BETRAY'){
#      $self->sight_of_player($player);
#    }
#  }
}

# writes 
sub sight_of_player{
  my ($self,$player) = @_;

  my $map = HexTorus->new($self->get_size());

  my $selfmap = $self->get_map();

  for my $row (@$selfmap){
    my ($loc,$ter,$own,$occ,$temple) = @$row;
    next if $self->{-sight_map}->{$loc};

    if($own == $player or $occ == $player
       or @{$self->own_in_mobile($self->{-game},$loc,$player)}){
      $self->{-sight_map}->{$loc} = 1;
      my $location = Location->from_string($loc);
      my $dist = 1;
      $dist = 2 if $ter eq 'MOUNTAIN';
      my @neighbours = $map->distant_neighbours($location,$dist);
      for my $l (@neighbours){
	$self->{-sight_map}->{$l->to_string()} = 1;
      }
    }
  }
}

1;
