#!/usr/local/bin/perl -w
##########################################################################
#
#   Copyright (c) 2003 Aymargeddon Development Team
#
#   This file is part of "Last days of Aymargeddon" 
#
#   Aymargeddon is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2 of the License, or (at your option)
#   any later version.
#
#   Aymargeddon is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#   more details.
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc., 675
#   Mass Ave, Cambridge, MA 02139, USA. 
#
###########################################################################
#

#
#
# checks the integrity of the database.
#
# usage: ./check.pl [-l] [list of checks]
# no args: do all checks
# -l: lists all avaiable checks
# -h: help
# list of checks: do only this checks

use strict;
$|=1;
use DBI;
use Data::Dumper;
use POSIX qw(floor ceil);
use Term::ReadLine;

use FROGS::HexTorus;
use FROGS::Check;

my $db_source = 'mysql:RAGNAROEK';
my $db_user = 'aymargeddon';
my $db_passwd = 'ragnaroek';

#
# Aymargeddon-specific behaviour
#

# TODO: should also check, if the location is valid in this map
my $location_wellformed_check = sub {
  my $loc = shift;
  my $wf = Location::is_wellformed($loc);
  print "$loc is in bad form! " unless $wf;
  return $wf;
};

my $no_earthling_collision_unless_fight = sub {
  print "check not available\n";
};


# list of checks
# every check consists of ID,behaviour
# behaviour is one of A_IN_B, LOGIK or UNIVERSAL

my $check = {
	     -GAME_EXISTS_FOR_MAP =>
	     ['A_IN_B', ['MAP','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_ALLIANCE =>
	     ['A_IN_B', ['ALLIANCE','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_COMMAND =>
	     ['A_IN_B', ['COMMAND','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_GOD =>
	     ['A_IN_B', ['GOD','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_GOD =>
	     ['A_IN_B', ['GOD','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_MESSAGE =>
	     ['A_IN_B', ['MESSAGE','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_ROLE =>
	     ['A_IN_B', ['ROLE','GAME','GAME','GAME']],
	     -GAME_EXISTS_FOR_MOBILE =>
	     ['A_IN_B', ['MOBILE','GAME','GAME','GAME']],

	     -LOCATION_WELLFORMED_IN_MAP =>
	     ['LOGIC',['MAP',['LOCATION'],$location_wellformed_check]],
	     -LOCATION_WELLFORMED_IN_MOBILE =>
	     ['LOGIC',['MOBILE',['LOCATION'],$location_wellformed_check]],
	     -ARRIVAL_WELLFORMED_IN_GOD =>
	     ['LOGIC',['GOD',['ARRIVAL'],$location_wellformed_check]],
	     -LAST_TEMPLE_WELLFORMED_IN_GAME =>
	     ['LOGIC',['GAME',['LAST_TEMPLE'],$location_wellformed_check]],

	     -NO_EARTHLING_COLLISION_UNLESS_FIGHT =>
	     ['UNIVERSAL', $no_earthling_collision_unless_fight],

	    };

my $checker = Check->new($db_source,$db_user,$db_passwd);
$checker->check_all($check);
	

