#!/usr/local/bin/perl -w
##########################################################################
#
#   Copyright (c) 2003 Aymargeddon Development Team
#
#   This file is part of "Last days of Aymargeddon" 
#
#   Aymargeddon is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2 of the License, or (at your option)
#   any later version.
#
#   Aymargeddon is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#   more details.
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc., 675
#   Mass Ave, Cambridge, MA 02139, USA. 
#
###########################################################################
#

#
#
# creates the world to destroy
#
# usage: ./create.pl [number of eartlings]
#
#

# TODO: use FROGS/DataBase.pm instead of DBI.pm

use strict;

use DBI;
use Data::Dumper;
use POSIX qw(floor ceil);
use Term::ReadLine;

use FROGS::HexTorus;

my $earthlings = $ARGV[0] || 10; # number of earthlings
my $n = ceil(sqrt($earthlings * 7)); # size of world

my $map = new HexTorus($n);

#my $loc = new Location(2,2);
#my @dn = $map->distant_neighbours($loc,2);
#print Dumper \@dn;
#exit;

my $db_source = 'mysql:RAGNAROEK';
my $db_user = 'aymargeddon';
my $db_passwd = 'ragnaroek';
my $db_table = 'MAP';

my $ascii = {
    -pole => '@',
    -city => '+',
    -home => '#',
    -water => '~',
    -island => '',
    -mountain => '^',
    -plain => '\'',
};

# create manapole
my $pole_count = floor(sqrt($n));
my $min_pole_distance = ceil($n/2);
# my $min_pole_distance = 2;
my $water_count = floor($n*$n/2);
my $island_count = floor($water_count/3);
my $min_home_distance = 2;
my $city_count = $earthlings * 4;

print "Earthlings: $earthlings\n". 
    "Gods: " . ($earthlings/2) . "\n".
    "Size of world: $n\n".
    "Number of poles: $pole_count\n".
    "Minimum pole distance: $min_pole_distance\n".
    "Water: $water_count\n".
    "Islands: $island_count\n".
    "Citys: $city_count\n".
    "Plain: " . ( ($n * $n * 2) - ($water_count + $pole_count +
				   $earthlings  + $city_count)) . "\n";

while(1){
    my @pole = distribute($pole_count,$min_pole_distance);

    my @water = flood($water_count,@pole);

    my @islands = vulcano($island_count, \@water, \@pole);

    my @mountains = distribute($earthlings,$min_home_distance,0,@water);

    my @home = distribute($earthlings,$min_home_distance,\@mountains,@water);

    my @not_cities = (@water,@pole,@mountains,@home);

    my @cities = build($city_count,\@not_cities);

    print $map->write_string($ascii->{-plain},"Plain",
			     [\@pole,$ascii->{-pole},"Pole"],
			     [\@water,$ascii->{-water},"Water"],
			     [\@islands,$ascii->{-island},"Islands"],
			     [\@home,$ascii->{-home},"Home"],
			     [\@mountains,$ascii->{-mountain},"Mountains"],
			     [\@cities,$ascii->{-city},"Cities"]);

    my $t = Term::ReadLine->new('test');

    my $r = $t->readline('Wrote this world to database? (y/n/q)');

    if($r =~ /y/i){
	
	my $dbh = DBI->connect("dbi:$db_source",$db_user,$db_passwd,
			       {'RaiseError' => 1,
				'AutoCommit' => 0}
				);

	my $table = $dbh->selectall_arrayref("select GAME from GAME");
	my @game_ids = sort {$a <=> $b} (map {$_->[0]} @$table);

	my $game = 1;
	for my $try (@game_ids){
	    last if $game < $try;
	    $game = $try + 1;
	}

	print "Erste Freie Game ID: $game\n";

	$map->write_db($dbh,$game,"PLAIN",
		       [\@pole,"POLE"],
		       [\@water,"WATER"],
		       [\@islands,"ISLE"],
		       [\@home,"CITY"],
		       [\@mountains,"MOUNTAIN"],
		       [\@cities,"CITY"]);

	# mark home citys and holy locations
	for my $home (@home, @mountains){
	    my $locstring = $dbh->quote($home->to_string());
	    my $home_command = "UPDATE MAP SET HOME = -1 ".
		"where GAME = $game AND LOCATION = $locstring";

	    # print $home_command."\n";
	    $dbh->do($home_command);
	}

	$dbh->commit();
	$dbh->disconnect();
	exit;
    }elsif($r =~ /q/i){
	exit;
    }
}

sub build{
    my ($count,$exclude) = @_;

    my %ex;
    if($exclude){
	for my $f (@$exclude){
	    $ex{$f->to_string()} = 1;
	}
    }
    my @citys;

    while($count){
	my $r = $map->random();
	unless(exists $ex{$r->to_string()}){
	    push @citys, $r;
	    $count--;
	}
    }
    return @citys;
}

sub vulcano{
    my ($count, $w,$p) = @_;
    my @w = @$w;
    my @p = @$p;

    my @islands;

    while($count){
	my $water_field = floor(rand($#w) + 1);
	my $wf = $w[$water_field];

	my $to_close = 0;
	for my $p (@p){
	  $to_close = 1 if($map->distance($p,$wf) < 2);
	}
	next if $to_close;

	splice @w,$water_field,1;
	push @islands, $wf;
	$count--;
    }
    return @islands;
}

sub flood{
    my ($count, @w) = @_;
    # print "flood($count,@w)\n";

    # flood neighbours
    my @new_w;
    for my $pole (@w){
	push @new_w, $map->neighbours($pole);
    }
    $count -= $#w + 1;
    return @w unless $count > 0;

    my %w;
    for my $w (@w,@new_w){
	$w{$w->to_string()} = 1;
    }
    @w = @new_w;

    while ($count){
	# print "$count ";
	# get a random water field
	my $water_field = floor(rand($#w) + 1);
	my $wf = $w[$water_field];
	# get neigbours without water 
	my @nb = $map->neighbours($wf);
	my @nb_new;
	for my $nb (@nb){
	    unless(exists $w{$nb->to_string()}){
		push @nb_new, $nb;
	    }
	}
	@nb = @nb_new;

	if ($#nb > 0){
	    # get a random neigbour
	    my $rn = floor(rand($#nb) + 1);
	    my $rnr = $nb[$rn];
	    push @w,$rnr;
	    $w{$rnr->to_string()} = 1;
	    $count --;
	}
    }
    return @w;
}

# distributes count locations at the map in minimal distance from
# fields not at exclude
# TODO: can get into an endless loop.
# TODO: should be in Map.pm. should be without endless loop
# TODO: should incorporate build() and vulcano()
sub distribute{ 
    my ($count,$dist,$exclude,@fields) = @_;

    my @return_fields;

    my %ex;
    if($exclude){
	for my $f (@$exclude){
	    $ex{$f->to_string()} = 1;
	}
    }

    while($count){
	my $r = $map->random();
	my $valid = 1;
	if($valid){
	    for my $xy (@fields){
		my $d = $map->distance($r,$xy);	
		my ($x,$y) = $xy->get();

		if($d < $dist or exists $ex{$r->to_string()}){
		    $valid = 0;
		    last;
		}
	    }
	}
	
	if($valid){
	    push @fields, $r;
	    push @return_fields, $r;
	    $count--;
	}
    }
    return @return_fields;
}




